from persiantools.jdatetime import JalaliDate


def date_fix(start: str=None, end: str=None) -> (str, str):
    new_start = None
    new_end = None
    if start is not None and not '-' in start:
        start = start[:4] + "-" + start[4:6] + "-" + start[6:]
    if end is not None and not '-' in end:
        end = end[:4] + "-" + end[4:6] + "-" + end[6:]

    if start is not None:
        two_left_char_start = start[:2]
        if two_left_char_start in ["13", "14", "15"]:
            new_start = JalaliDate.to_gregorian(JalaliDate.fromisoformat(start)).isoformat()
        else:
            new_start = start
    if end is not None:
        two_left_char_end = end[:2]
        if two_left_char_end in ["13", "14", "15"]:
            new_end = JalaliDate.to_gregorian(JalaliDate.fromisoformat(end)).isoformat()
        else:
            new_end = end
    else:
        new_end = end
    return new_start, new_end



import requests
import warnings
import pandas as pd
from algotik_tse.settings import Settings
from algotik_tse.core.search import search_stock
warnings.simplefilter(action='ignore', category=FutureWarning)

settings = Settings()


def stockdetail(stock):
    """
    Get all symbol detail
    :param stock: name of stock
                  Default value is 'شتران'.

    :return: pandas dataframe
    """
    web_id = search_stock(search_txt=stock)
    if len(web_id) != 0:
        if web_id[-5:] == "index":
            print("This is an index, no information found!")
            return None
        else:
            detail = requests.get(settings.url_detail.format(web_id), headers=settings.headers)
            if detail.status_code == 200:
                df = pd.read_html(detail.content, encoding='utf8')[0]
                df.rename(columns={0: 'key', 1: 'value'}, inplace=True)
                df.set_index('key', drop=True, inplace=True)
                df.loc['id'] = str(web_id)
                return df
            else:
                print("Connection Error!!!")
                return None
    else:
        print("Stock Not Found, Please try again ...")
        return None



import requests
import warnings
import pandas as pd
from persiantools import characters
from algotik_tse.settings import Settings

warnings.simplefilter(action='ignore', category=FutureWarning)

settings = Settings()


def stocklist(bourse=True, farabourse=True, payeh=True, haghe_taqadom=False, sandogh=False, output="dataframe",
              progress=True):
    """
    Get all symbols in bourse, farabourse, payeh, haghe_taqadom from tsetmc
    :param bourse:          if True you can get all bourse symbols in output.
                            Default value is True.
    :param farabourse:      if True you can get all farabourse symbols in output.
                            Default value is True.
    :param payeh:           if True you can get all payeh symbols in output.
                            Default value is True.
    :param haghe_taqadom:   if True you can get all payeh symbols in output.
                            Default value is False.
    :param sandogh:         if True you can get all sandogh symbols in output.
                            Default value is False.
    :param output:          you can choose output format between :
                                dataftame(stock with detal) and
                                list(stocks name only)
                            Default value is False.
    :param progress:        if True, show progress and report in console.
                            Default value is True.

    :return: pandas dataframe or list
    """
    if not bourse and not farabourse and not payeh and not haghe_taqadom and not sandogh:
        print("You should select at least one market!")
        return None
    else:
        markets_text_list = []
        if bourse:
            markets_text_list.append('bourse')
        if farabourse:
            markets_text_list.append('farabourse')
        if payeh:
            markets_text_list.append('payeh')
        if haghe_taqadom:
            markets_text_list.append('haghe taqadom')
        if sandogh:
            markets_text_list.append('sandogh')

        markets_text = ""
        for n in range(len(markets_text_list)):
            if n == 0:
                markets_text += markets_text_list[n]
            else:
                markets_text += (", " + markets_text_list[n])
        if progress:
            print("Getting all of {} symbols...".format(markets_text), flush=True)

        try:
            req = requests.get(settings.url_stock_list)
            df = pd.read_html(req.content, encoding='utf8', extract_links='body')[0]

            df.rename(columns={'کد 12 رقمی نماد [Instrument ISIN]': 'instrument_isin',
                               'کد 4 رقمی شرکت [Company Code]': 'company_code',
                               'نام انگلیسی [English Name]': 'english_name',
                               'کد 12 رقمی شرکت [Company ISIN]': 'company_isin', 'بازار': 'market',
                               'گروه صنعت': 'industry_group', 'نوع [Type]': 'type'}, inplace=True)
            df['name'] = df['نماد [Name]'].str[0]
            df['instrument_isin'] = df['instrument_isin'].str[0]
            df['company_code'] = df['company_code'].str[0]
            df['english_name'] = df['english_name'].str[0]
            df['company_isin'] = df['company_isin'].str[0]
            df['market'] = df['market'].str[0]
            df['industry_group'] = df['industry_group'].str[0]
            df['instrument_id'] = df['نماد [Name]'].str[-1]
            df['instrument_id'] = df['instrument_id'].str.strip().str.split('=').str[-1]
            df['instrument_id'] = df['instrument_id'].str.split('/').str[-1]

            df.drop(columns=['نماد [Name]', 'type'], inplace=True)

            def split_symbol(group):
                dd = group.split('(')
                if 'هلد' in dd[1] or 'سهامی' in dd[1]:
                    xx = dd[2].split(")")[0].strip()
                else:
                    xx = dd[1].split(")")[0].strip()
                return xx

            allowed_markets = []

            group_not_allowed = ['اوراق حق تقدم استفاده از تسهيلات مسكن']
            not_all = ['بازار سوم فرابورس', '-', 'بازار عادي آتي']

            df_final = pd.DataFrame()

            if bourse or farabourse or payeh:
                allowed_pre_instrument_isin = ['IRO1', 'IRO2', 'IRO3', 'IRO4', 'IRO5', 'IRO7']
                if bourse:
                    allowed_markets.extend(
                        ['بازار دوم بورس', 'بازار اول (تابلوي فرعي) بورس', 'بازار اول (تابلوي اصلي) بورس'])
                if farabourse:
                    allowed_markets.extend(['بازار اول فرابورس', 'شرکتهاي کوچک و متوسط فرابورس', 'بازار دوم فرابورس'])
                if payeh:
                    allowed_markets.extend(
                        ['بازار پايه نارنجي فرابورس', 'بازار پایه قرمز فرابورس', 'بازار پايه زرد فرابورس',
                         'بازار پايه قرمز فرابورس'])

                df_filtered = df[(df['instrument_isin'].str.slice(stop=4).isin(allowed_pre_instrument_isin)) & (
                    ~df['industry_group'].isin(group_not_allowed)) & (df['instrument_isin'].str.slice(-1) == '1')]
                df_filtered = df_filtered[~df_filtered['market'].isin(not_all)]
                df_filtered = df_filtered[df_filtered['market'].isin(allowed_markets)]
                df_final = pd.concat([df_final, df_filtered])

            if haghe_taqadom:
                allowed_pre_instrument_isin = ['IRR1', 'IRR3', 'IRR5', 'IRR7']
                df_filtered = df[(df['instrument_isin'].str.slice(stop=4).isin(allowed_pre_instrument_isin)) & (
                    ~df['industry_group'].isin(group_not_allowed)) & (df['instrument_isin'].str.slice(-1) == '1')]
                df_filtered = df_filtered[~df_filtered['market'].isin(not_all)]
                df_final = pd.concat([df_final, df_filtered])

            if sandogh:
                allowed_pre_instrument_isin = ['IRT1', 'IRT3', 'IRTE', 'IRTK']
                df_filtered = df[(df['instrument_isin'].str.slice(stop=4).isin(allowed_pre_instrument_isin) & (
                    df['instrument_isin'].str.slice(-1) == '1'))]
                df_final = pd.concat([df_final, df_filtered])
        except:
            df_final = pd.DataFrame()

        if df_final.shape[0] != 0:
            df_final['name'] = df_final['name'].str.strip().apply(characters.ar_to_fa)
            df_final['symbol'] = df_final['name'].apply(split_symbol)
            df_final.set_index('symbol', drop=True, inplace=True)
            df_final['name'] = df_final['name'].str.split('(').str[0]
            df_final['name'] = df_final['name'].str.replace('\u200c', ' ').str.strip()
            df_final['market'] = df_final['market'].str.strip().apply(characters.ar_to_fa)
            df_final['market'] = df_final['market'].str.replace('\u200c', ' ').str.strip()
            df_final['industry_group'] = df_final['industry_group'].str.strip().apply(characters.ar_to_fa)
            df_final['industry_group'] = df_final['industry_group'].str.replace('\u200c', ' ').str.strip()
            df_final = df_final.loc[:,
                       ['name', 'instrument_isin', 'english_name', 'company_code', 'company_isin', 'market',
                        'industry_group', 'instrument_id']]

            if output == "dataframe":
                if progress:
                    print("The dataframe of all {} symbols is ready!".format(markets_text))
                return df_final
            elif output == "list":
                if progress:
                    print("The list of all {} symbols is ready!".format(markets_text))
                symbols_list = df_final.index.to_list()
                return symbols_list
            else:
                print("You should select output only in 'dataframe' or 'list'!!!")
                return None
        else:
            print("Connection Error!!!")
            return None



"""
version 0.2.4
Main module code by @Python4finance
New version code by Mohsen Alipour alipour@algotik.ir
last Edit: 2023-10-01
* Base: old.tsetmc.com

Sample:
import algotik_tse as att

att.stockdetail(stock='شبندر')
you get all detail information of the stock!

att.stocklist(bourse=True, farabourse=True, payeh=True, haghe_taqadom=False, output="dataframe"):
show all stock data in dataframe or list format (you can choose bourse, farabourse, payeh and haghe_taqadom)

att.stock(stock="شبندر", start=None, end=None, values=0, tse_format=False, auto_adjust=True, output_type="standard",
 progress=True, date_format="jalali", save_to_file=False, multi_stock_drop=True, adjust_volume=False)
you can get historical price data, in customized way with set the function parameters!
"""

__author__ = """Mohsen Alipour"""
__email__ = 'alipour@algotik.ir'
__version__ = '0.2.4'

# import algotik_tse.settings
# from algotik_tse.core.search import search_stock
from algotik_tse.core.stock_detail import stockdetail
from algotik_tse.core.stock_list import stocklist
from algotik_tse.core.stock import stock, stock_RL



import io
import datetime
import requests
import warnings
import pandas as pd
from persiantools import characters
from persiantools.jdatetime import JalaliDate

from algotik_tse.settings import Settings
from algotik_tse.core.search import search_stock
from algotik_tse.core.helper import date_fix

warnings.simplefilter(action='ignore', category=FutureWarning)

settings = Settings()


def stock(stock="", start=None, end=None, values=0, tse_format=False, auto_adjust=True, output_type="standard",
          date_format="jalali", progress=True, save_to_file=False, multi_stock_drop=True, adjust_volume=False):
    """
    Get symbol or symbols price history from tsetmc
    :param stock:           stock name in persian, or a list of stock in
                                persian (['شتران', 'آریا'])
                            Default value is 'شتران'.
    :param start:           you can choose strat date (from) to get historical price.
                            enter date in jalali only in isoformat ('1401-10-01')
                                and gregorian in isoformat("2022-12-22") or
                                ("2022") or ("2022-12")
                            Default value is None.
                            if start=None and end=None, value can apply to price data.
    :param end:             you can choos end date (to) to get historical price.
                            enter date in jalali only in isoformat ('1401-10-01')
                                and gregorian in isoformat ("2022-12-22") or
                                ("2022") or ("2022-12")
                            Default value is None.
                            if start=None and end=None, 'values' can apply to price data.
    :param values:          Specifies the number of price data since today
                            Default value is 225.
                            'values' can be applied when start=None and end=None.
    :param tse_format:      if True you can get all historical data in tse .csv format
                            Default value is False.
                            if tse_format=True, ignore auto_adjust, output_type,
                                date_format, adjust_volume!
    :param auto_adjust:     if True, adjust all OHLC price, with stock splits and dps.
                            Default value is True.
                            if False, show 'Adj Close' column in output
    :param output_type:     you can choose between 'standard' and 'complete'.
                            Default value is 'standard'.
                            if output_type='standard', you get OHLC and Volume
                                (and 'Adj Close' if auto_adjust=False) in output.
                            if output_type='complete', you get OHLC and Volume
                                (and 'Adj Close' if auto_adjust=False) and 'No.',
                                'Value', 'Weekday', 'Ticker' in output.
    :param date_format:     you can choose between 'jalali' and 'gregorian' and 'both'.
                            Default value is 'jalali'.
                            if date_format='jalali', you get historical price with
                                jalali date index and 'Weekday_fa' in complete mode
                                in output.
                            if output_type='gregorian', you get historical price
                                with gregorian date index and 'Weekday' in complete
                                mode in output.
                            if output_type='both', you get historical price with
                                gregorian date index and 'J-Date'(jalali date),
                                'Weekday', 'Weekday_fa' in complete mode in output.
    :param progress:        if True, show progress and report in console.
                            Default value is True.
    :param save_to_file:    if True, save stock(s) historical data with customized
                                columns in .csv format.
                            Default value is False.
                            the file name is 'stock.csv' in same root that
                                tsemodule6 is there. for example: 'شتران.csv'
    :param multi_stock_drop:if True, when you enter stocks list, it will delete
                                rows of none data (dropna) in combined historical df.
                            Default value is True.
    :param adjust_volume:   if True, when output_type='complete' you can get
                                'Adj Volume' in output.
                            Default value is False.

    :return: pandas dataframe or None
    """
    def _get_stock(stock_name, mstart, mend, mvalues, mtse_format, mauto_adjust, moutput_type, mdate_format, ):
        web_id = search_stock(search_txt=stock_name)
        _price_base_url = settings.url_price_history
        isIndex = False
        if web_id[-5:] == "index":
            web_id = web_id[:-5]
            _price_base_url = settings.url_index_history
            isIndex = True
        new_start, new_end = date_fix(start=mstart, end=mend)
        if new_start is not None or new_end is not None:
            mvalues = 0
        if isIndex:
            index_names = {'32097828799138957': 'Overall Index',
                           '67130298613737946': 'Total Equal Weighted Index',
                           '5798407779416661': 'Total Price Index',
                           '8384385859414435': 'Equal Weighted Price Index',
                           '49579049405614711': 'Free Float Index',
                           '62752761908615603': 'OTC Main Board Index',
                           '71704845530629737': 'OTC Secondary Board Index',
                           '43754960038275285': 'Industry Index',
                           '10523825119011581': 'Top 30 Index',
                           '46342955726788357': 'Top 50 Index'}
            try:
                data = {"<TICKER>": [], "<DTYYYYMMDD>": [], "<FIRST>": [], "<HIGH>": [], "<LOW>": [], "<CLOSE>": [],
                        "<VOL>": [], "<PER>": [], "<OPEN>": [], "<LAST>": []}
                fopen = requests.get(_price_base_url.format(web_id), headers=settings.headers).text.split(";")
                for dt in fopen:
                    dts = dt.split(",")
                    data["<TICKER>"].append(index_names[web_id])
                    date_iso = dts[0][:4] + "-" + dts[0][4:6] + "-" + dts[0][6:]
                    data["<DTYYYYMMDD>"].append(datetime.date.fromisoformat(date_iso))
                    data["<FIRST>"].append(float(dts[3]))
                    data["<HIGH>"].append(float(dts[1]))
                    data["<LOW>"].append(float(dts[2]))
                    data["<CLOSE>"].append(float(dts[6]))
                    data["<VOL>"].append(float(dts[5]))
                    data["<PER>"].append('D')
                    data["<OPEN>"].append(float(dts[3]))
                    data["<LAST>"].append(float(dts[4]))

                df = pd.DataFrame(data, columns=data.keys(), index=pd.DatetimeIndex(data["<DTYYYYMMDD>"]))
                df.index.names = ['<DTYYYYMMDD>']
                df.drop(columns=['<DTYYYYMMDD>'], inplace=True)

                if mvalues is not None or mstart is not None or mend is not None:
                    if mvalues != 0:
                        df = df.iloc[-mvalues:]
                    else:
                        if new_end is None:
                            df = df.loc[new_start:]
                        else:
                            df = df.loc[new_start:new_end]

                if mtse_format:
                    return df
                else:
                    df.index.rename("Date_base", inplace=True)
                    df.drop(["<TICKER>", "<PER>", "<OPEN>"], axis=1, inplace=True)
                    df.rename(columns={"<FIRST>": "Open", "<HIGH>": "High", "<LOW>": "Low", "<CLOSE>": "Adj Close",
                                       "<VOL>": "Volume", "<LAST>": "Close"}, inplace=True)
                    df = df.loc[:, ["Open", "High", "Low", "Close", "Adj Close", "Volume", ]]
                    df["Date"] = df.index
                    df["J-Date"] = df["Date"]
                    df['J-Date'] = df['J-Date'].apply(lambda x: x.strftime("%Y-%m-%d"))
                    df['J-Date'] = df['J-Date'].apply(
                        lambda x: JalaliDate.to_jalali(datetime.datetime.fromisoformat(x)).isoformat())
                    df["Weekday_No"] = df["Date"].dt.weekday
                    df["Weekday"] = df["Weekday_No"].apply(lambda x: settings.en_weekdays[x])
                    df["Weekday_fa"] = df["Weekday_No"].apply(lambda x: settings.fa_weekdays[x])
                    df.drop("Weekday_No", axis=1, inplace=True)
                    df['Ticker'] = stock_name

                    if mauto_adjust:
                        df.drop("Adj Close", axis=1, inplace=True)
                        if mdate_format == "jalali":
                            df.set_index("J-Date", drop=True, inplace=True)
                            df.drop(columns=["Date", "Weekday"], inplace=True)
                        elif mdate_format == "gregorian":
                            df.set_index("Date", drop=True, inplace=True)
                            df.drop(columns=["J-Date", "Weekday_fa"], inplace=True)
                        elif mdate_format == "both":
                            df.set_index("Date", drop=True, inplace=True)
                        else:
                            print("please select date_format between 'jalali', 'gregorian ', 'both' ")
                            return None
                        if moutput_type == "standard":
                            df = df.loc[:, ['Open', 'High', 'Low', 'Close', 'Volume']]
                        elif moutput_type == "complete":
                            pass
                        else:
                            print("output_type should select between 'standard' or 'complete'")
                            return None
                    else:
                        if mdate_format == "jalali":
                            df.set_index("J-Date", drop=True, inplace=True)
                            df.drop(columns=["Date", "Weekday"], inplace=True)
                        elif mdate_format == "gregorian":
                            df.set_index("Date", drop=True, inplace=True)
                            df.drop(columns=["J-Date", "Weekday_fa"], inplace=True)
                        elif mdate_format == "both":
                            df.set_index("Date", drop=True, inplace=True)
                        else:
                            print("please select date_format between 'jalali', 'gregorian', 'both' ")
                            return None
                        if moutput_type == "standard":
                            df = df.loc[:, ['Open', 'High', 'Low', 'Close', 'Adj Close', 'Volume']]
                        elif moutput_type == "complete":
                            pass
                        else:
                            print("output_type should select between 'standard' or 'complete'")
                            return None
                    return df
            except:
                print("Connection Error!")
                return None
        else:
            try:
                fopen = requests.get(_price_base_url.format(web_id), headers=settings.headers).content
                df = pd.read_csv(io.StringIO(fopen.decode("utf-8")), index_col="<DTYYYYMMDD>", parse_dates=True)
                df = df[::-1]
                if mvalues is not None or mstart is not None or mend is not None:
                    if mvalues != 0:
                        df = df.iloc[-mvalues:]
                        pass
                    else:
                        if new_end is None:
                            df = df.loc[new_start:]
                        else:
                            df = df.loc[new_start:new_end]

                if mtse_format:
                    return df
                else:
                    df.index.rename("Date_base", inplace=True)
                    df.drop(["<TICKER>", "<PER>"], axis=1, inplace=True)
                    df.rename(columns={"<FIRST>": "Open", "<HIGH>": "High", "<LOW>": "Low", "<CLOSE>": "Final",
                                       "<VALUE>": "Value", "<VOL>": "Volume", "<OPENINT>": "No.",
                                       "<OPEN>": "Yesterday-Final", "<LAST>": "Close"}, inplace=True)
                    df = df.loc[:,
                         ["Open", "High", "Low", "Close", "Final", "Volume", "Yesterday-Final", "No.", "Value"]]
                    df["Final+1"] = df["Final"].shift(1)
                    df['pos'] = df.apply(
                        lambda x: x['Yesterday-Final'] if (
                            (x['Yesterday-Final'] != 0) and (x['Yesterday-Final'] != 1000))
                        else (x['Yesterday-Final'] if (pd.isnull(x['Final+1'])) else x['Final+1']), axis=1)
                    df['Yesterday-Final'] = df['pos']
                    df.drop(columns=['Final+1', 'pos'], inplace=True)
                    df['coef'] = (df['Yesterday-Final'].shift(-1) / df['Final']).fillna(1.)
                    df['Adj coef'] = df.iloc[::-1]['coef'].cumprod().iloc[::-1]
                    df['Adj Vol coef'] = 1 / df['Adj coef']
                    df['Adj Close'] = (df['Close'] * df['Adj coef']).apply(lambda x: int(x))
                    df['Adj Open'] = (df['Open'] * df['Adj coef']).apply(lambda x: int(x))
                    df['Adj High'] = (df['High'] * df['Adj coef']).apply(lambda x: int(x))
                    df['Adj Low'] = (df['Low'] * df['Adj coef']).apply(lambda x: int(x))
                    df['Adj Final'] = (df['Final'] * df['Adj coef']).apply(lambda x: int(x))
                    df['Adj Volume'] = (df['Volume'] * df['Adj Vol coef']).apply(lambda x: int(x))
                    df.drop(columns=['coef', 'Adj coef', 'Adj Vol coef'], inplace=True)
                    df["Date"] = df.index
                    df["J-Date"] = df["Date"]
                    df['J-Date'] = df['J-Date'].apply(lambda x: x.strftime("%Y-%m-%d"))
                    df['J-Date'] = df['J-Date'].apply(
                        lambda x: JalaliDate.to_jalali(datetime.datetime.fromisoformat(x)).isoformat())
                    df["Weekday_No"] = df["Date"].dt.weekday
                    df["Weekday"] = df["Weekday_No"].apply(lambda x: settings.en_weekdays[x])
                    df["Weekday_fa"] = df["Weekday_No"].apply(lambda x: settings.fa_weekdays[x])
                    df.drop("Weekday_No", axis=1, inplace=True)
                    df['Ticker'] = stock_name
                    if mauto_adjust:
                        if adjust_volume:
                            df = df.loc[:,
                                 ['Adj Open', 'Adj High', 'Adj Low', 'Adj Close', 'Adj Final', 'Volume', 'Adj Volume',
                                  'No.', 'Value', 'Date', 'J-Date', 'Weekday', 'Weekday_fa', 'Ticker']]
                        else:
                            df = df.loc[:, ['Adj Open', 'Adj High', 'Adj Low', 'Adj Close', 'Adj Final', 'Volume',
                                            'No.', 'Value', 'Date', 'J-Date', 'Weekday', 'Weekday_fa', 'Ticker']]
                        df.rename(columns={'Adj Open': 'Open', 'Adj High': 'High', 'Adj Low': 'Low',
                                           'Adj Close': 'Close', 'Adj Final': 'Final'}, inplace=True)
                        if mdate_format == "jalali":
                            df.set_index("J-Date", drop=True, inplace=True)
                            df.drop(columns=["Date", "Weekday"], inplace=True)
                        elif mdate_format == "gregorian":
                            df.set_index("Date", drop=True, inplace=True)
                            df.drop(columns=["J-Date", "Weekday_fa"], inplace=True)
                        elif mdate_format == "both":
                            df.set_index("Date", drop=True, inplace=True)
                        else:
                            print("please select date_format between 'jalali', 'gregorian ', 'both' ")
                            return None
                        if moutput_type == "standard":
                            df = df.loc[:, ['Open', 'High', 'Low', 'Close', 'Volume']]
                        elif moutput_type == "complete":
                            pass
                        else:
                            print("output_type should select between 'standard' or 'complete'")
                            return None
                    else:
                        if adjust_volume:
                            df = df.loc[:,
                                 ['Open', 'High', 'Low', 'Close', 'Final', 'Adj Close', 'Volume', 'Adj Volume',
                                  'No.', 'Value', 'Date', 'J-Date', 'Weekday', 'Weekday_fa', 'Ticker']]
                        else:
                            df = df.loc[:, ['Open', 'High', 'Low', 'Close', 'Final', 'Adj Close', 'Volume',
                                            'No.', 'Value', 'Date', 'J-Date', 'Weekday', 'Weekday_fa', 'Ticker']]
                        if mdate_format == "jalali":
                            df.set_index("J-Date", drop=True, inplace=True)
                            df.drop(columns=["Date", "Weekday"], inplace=True)
                        elif mdate_format == "gregorian":
                            df.set_index("Date", drop=True, inplace=True)
                            df.drop(columns=["J-Date", "Weekday_fa"], inplace=True)
                        elif mdate_format == "both":
                            df.set_index("Date", drop=True, inplace=True)
                        else:
                            print("please select date_format between 'jalali', 'gregorian', 'both' ")
                            return None
                        if moutput_type == "standard":
                            df = df.loc[:, ['Open', 'High', 'Low', 'Close', 'Adj Close', 'Volume']]
                        elif moutput_type == "complete":
                            pass
                        else:
                            print("output_type should select between 'standard' or 'complete'")
                            return None
                    return df

            except:
                print("Stock Not Found!")
                return None

    if stock == "":
        stock = "شتران"
        if progress:
            print("1/1: Getting historical price of {}".format(stock))
        stock = characters.ar_to_fa(stock).strip("\u200c").strip()
        df = _get_stock(stock_name=stock, mstart=start, mend=end, mvalues=values, mtse_format=tse_format,
                        mauto_adjust=auto_adjust, moutput_type=output_type, mdate_format=date_format)
        if progress and df is not None:
            print("1/1: Completed!")
        if save_to_file and df is not None:
            if progress:
                print("Saving to file: {}.csv".format(stock))
            df.to_csv(stock + ".csv", encoding="utf-8-sig")
        return df
    else:
        if isinstance(stock, str):
            if progress:
                print("1/1: Getting historical price of {}".format(stock))
            stock = characters.ar_to_fa(stock).strip("\u200c").strip()
            df = _get_stock(stock_name=stock, mstart=start, mend=end, mvalues=values, mtse_format=tse_format,
                            mauto_adjust=auto_adjust, moutput_type=output_type, mdate_format=date_format)
            if progress and df is not None:
                print("1/1: Completed!")
            if save_to_file and df is not None:
                if progress:
                    print("Saving to file: {}.csv".format(stock))
                df.to_csv(stock + ".csv", encoding="utf-8-sig")
            return df
        elif isinstance(stock, list):
            n = 1
            df_dict = {}
            file_name_str = ''
            for stk in stock:
                if progress:
                    print("{}/{}: Getting historical price of {}".format(n, len(stock), stk))
                stk = characters.ar_to_fa(stk).strip("\u200c").strip()
                df = _get_stock(stock_name=stk, mstart=start, mend=end, mvalues=values, mtse_format=tse_format,
                                mauto_adjust=auto_adjust, moutput_type=output_type, mdate_format=date_format)
                if df is not None:
                    file_name_str += "-" + stk
                    df_dict[stk] = df
                else:
                    print("{} not Found!".format(stk))
                n += 1
            if progress:
                print('{}/{} Completed!'.format(len(stock), len(stock)))

            if len(list(df_dict.keys())) == 0:
                print('None of the entered stocks exist!!')
                return None
            elif len(list(df_dict.keys())) == 1:
                df = df_dict[list(df_dict.keys())[0]]
                if save_to_file and df is not None:
                    if progress:
                        print("Saving to file: {}.csv".format(file_name_str[1:]))
                    df.to_csv(file_name_str[1:] + ".csv", encoding="utf-8-sig")
                return df
            else:
                df = pd.concat(df_dict, axis=1)
                if multi_stock_drop:
                    df.dropna(inplace=True)
                if save_to_file and df is not None:
                    if progress:
                        print("Saving to file: {}.csv".format(file_name_str[1:]))
                    df.to_csv(file_name_str[1:] + ".csv", encoding="utf-8-sig")
                return df


def stock_RL(stock="", start=None, end=None, values=0, tse_format=False, output_type="standard", date_format="jalali",
             progress=True, save_to_file=False, multi_stock_drop=True):
    """
    Get symbol or symbols real/legal history from tsetmc
    :param stock:           stock name in persian, or a list of stock in
                                persian (['شتران', 'آریا'])
                            Default value is 'شتران'.
    :param start:           you can choose strat date (from) to get historical real/legal.
                            enter date in jalali only in isoformat ('1401-10-01')
                                and gregorian in isoformat("2022-12-22") or
                                ("2022") or ("2022-12")
                            Default value is None.
                            if start=None and end=None, value can apply to real/legal data.
    :param end:             you can choos end date (to) to get historical real/legal.
                            enter date in jalali only in isoformat ('1401-10-01')
                                and gregorian in isoformat ("2022-12-22") or
                                ("2022") or ("2022-12")
                            Default value is None.
                            if start=None and end=None, 'values' can apply to real/legal data.
    :param values:          Specifies the number of price data since today
                            Default value is 225.
                            'values' can be applied when start=None and end=None.
    :param tse_format:      if True you can get all historical real/legal data in tse .csv format
                            Default value is False.
                            if tse_format=True, ignore output_type, date_format!
    :param output_type:     you can choose between 'standard' and 'complete'.
                            Default value is 'standard'.
                            if output_type='standard', you get real/legal without per capitas
                                or powers in output.
                            if output_type='complete', you get real/legal with per capitas
                                and powers in output
    :param date_format:     you can choose between 'jalali' and 'gregorian' and 'both'.
                            Default value is 'jalali'.
                            if date_format='jalali', you get historical real/legal with
                                jalali date index and 'Weekday_fa' in complete mode
                                in output.
                            if output_type='gregorian', you get historical real/legal
                                with gregorian date index and 'Weekday' in complete
                                mode in output.
                            if output_type='both', you get historical real/legal with
                                gregorian date index and 'J-Date'(jalali date),
                                'Weekday', 'Weekday_fa' in complete mode in output.
    :param progress:        if True, show progress and report in console.
                            Default value is True.
    :param save_to_file:    if True, save stock(s) historical real/legal data with customized
                                columns in .csv format.
                            Default value is False.
                            the file name is 'stock.csv' in same root that
                                tsemodule6 is there. for example: 'شتران.csv'
    :param multi_stock_drop:if True, when you enter stocks list, it will delete
                                rows of none data (dropna) in combined historical df.
                            Default value is True.

    :return: pandas dataframe or None
    """
    def _get_stock_RL(stock_name, mstart, mend, mvalues, mtse_format, moutput_type, mdate_format):
        web_id = search_stock(search_txt=stock_name)
        client_type_base_url = settings.url_client_type
        if web_id[-5:] == "index":
            print("{} is an index, Please enter a valid stock name!".format(stock_name))
            return None
        new_start, new_end = date_fix(start=mstart, end=mend)
        if new_start is not None or new_end is not None:
            mvalues = 0
        try:
            fopen = requests.get(client_type_base_url.format(web_id)).text.split(";")
            data = {"<DTYYYYMMDD>": [], "<TICKER>": [], "<N_BUY_REAL>": [], "<N_BUY_LEGAL>": [], "<N_SELL_REAL>": [],
                    "<N_SELL_LEGAL>": [],
                    "<VOL_BUY_REAL>": [], "<VOL_BUY_LEGAL>": [], "<VOL_SELL_REAL>": [], "<VOL_SELL_LEGAL>": [],
                    "<VAL_BUY_REAL>": [], "<VAL_BUY_LEGAL>": [], "<VAL_SELL_REAL>": [], "<VAL_SELL_LEGAL>": [],
                    "<PER>": []}

            for dt in fopen:
                dts = dt.split(",")
                date_iso = dts[0][:4] + "-" + dts[0][4:6] + "-" + dts[0][6:]
                data["<DTYYYYMMDD>"].append(datetime.date.fromisoformat(date_iso))
                data['<TICKER>'].append(stock_name)
                data['<N_BUY_REAL>'].append(int(dts[1]))
                data['<N_BUY_LEGAL>'].append(int(dts[2]))
                data['<N_SELL_REAL>'].append(int(dts[3]))
                data['<N_SELL_LEGAL>'].append(int(dts[4]))
                data['<VOL_BUY_REAL>'].append(int(dts[5]))
                data['<VOL_BUY_LEGAL>'].append(int(dts[6]))
                data['<VOL_SELL_REAL>'].append(int(dts[7]))
                data['<VOL_SELL_LEGAL>'].append(int(dts[8]))
                data['<VAL_BUY_REAL>'].append(int(dts[9]))
                data['<VAL_BUY_LEGAL>'].append(int(dts[10]))
                data['<VAL_SELL_REAL>'].append(int(dts[11]))
                data['<VAL_SELL_LEGAL>'].append(int(dts[12]))
                data['<PER>'].append('D')

            df = pd.DataFrame(data, columns=data.keys(), index=pd.DatetimeIndex(data["<DTYYYYMMDD>"]))
            df.index.names = ['<DTYYYYMMDD>']
            df.drop(columns=['<DTYYYYMMDD>'], inplace=True)

            df = df[::-1]
            if mvalues is not None or mstart is not None or mend is not None:
                if mvalues != 0:
                    df = df.iloc[-mvalues:]
                else:
                    if new_end is None:
                        df = df.loc[new_start:]
                    else:
                        df = df.loc[new_start:new_end]

            if mtse_format:
                return df
            else:
                df.index.rename("Date_base", inplace=True)
                df.drop(["<TICKER>", "<PER>"], axis=1, inplace=True)
                df.rename(columns={"<N_BUY_REAL>": "N_buy_real", "<N_BUY_LEGAL>": "N_buy_legal",
                                   "<N_SELL_REAL>": "N_sell_real", "<N_SELL_LEGAL>": "N_sell_legal",
                                   "<VOL_BUY_REAL>": "Vol_buy_real", "<VOL_BUY_LEGAL>": "Vol_buy_legal",
                                   "<VOL_SELL_REAL>": "Vol_sell_real", "<VOL_SELL_LEGAL>": "Vol_sell_legal",
                                   "<VAL_BUY_REAL>": "Val_buy_real", "<VAL_BUY_LEGAL>": "Val_buy_legal",
                                   "<VAL_SELL_REAL>": "Val_sell_real", "<VAL_SELL_LEGAL>": "Val_sell_legal"},
                          inplace=True)

                df['Per_capita_buy_real'] = round(df['Val_buy_real'] / df['N_buy_real'])
                df['Per_capita_sell_real'] = round(df['Val_sell_real'] / df['N_sell_real'])
                df['Per_capita_buy_legal'] = round(df['Val_buy_legal'] / df['N_buy_legal'])
                df['Per_capita_sell_legal'] = round(df['Val_sell_legal'] / df['N_sell_legal'])
                df['Power_real'] = round(df['Per_capita_buy_real'] / df['Per_capita_sell_real'], 3)
                df['Power_legal'] = round(df['Per_capita_buy_legal'] / df['Per_capita_sell_legal'], 3)

                df["Date"] = df.index
                df["J-Date"] = df["Date"]
                df['J-Date'] = df['J-Date'].apply(lambda x: x.strftime("%Y-%m-%d"))
                df['J-Date'] = df['J-Date'].apply(
                    lambda x: JalaliDate.to_jalali(datetime.datetime.fromisoformat(x)).isoformat())
                df["Weekday_No"] = df["Date"].dt.weekday
                df["Weekday"] = df["Weekday_No"].apply(lambda x: settings.en_weekdays[x])
                df["Weekday_fa"] = df["Weekday_No"].apply(lambda x: settings.fa_weekdays[x])
                df.drop("Weekday_No", axis=1, inplace=True)
                df['Ticker'] = stock_name
                df.fillna(value=0, inplace=True)

                if mdate_format == "jalali":
                    df.set_index("J-Date", drop=True, inplace=True)
                    df.drop(columns=["Date", "Weekday"], inplace=True)
                elif mdate_format == "gregorian":
                    df.set_index("Date", drop=True, inplace=True)
                    df.drop(columns=["J-Date", "Weekday_fa"], inplace=True)
                elif mdate_format == "both":
                    df.set_index("Date", drop=True, inplace=True)
                else:
                    print("please select date_format between 'jalali', 'gregorian ', 'both' ")
                    return None
                if moutput_type == "standard":
                    df = df.loc[:, ["N_buy_real", "N_buy_legal", "N_sell_real", "N_sell_legal", "Vol_buy_real",
                                    "Vol_buy_legal", "Vol_sell_real", "Vol_sell_legal", "Val_buy_real", "Val_buy_legal",
                                    "Val_sell_real", "Val_sell_legal", ]]
                elif moutput_type == "complete":
                    pass
                else:
                    print("output_type should select between 'standard' or 'complete'")
                    return None
                return df
        except:
            print("Stock Not Found!")
            return None

    if stock == "":
        stock = "شتران"
        if progress:
            print("1/1: Getting historical real/legal of {}".format(stock))
        stock = characters.ar_to_fa(stock).strip("\u200c").strip()
        df = _get_stock_RL(stock_name=stock, mstart=start, mend=end, mvalues=values, mtse_format=tse_format,
                           moutput_type=output_type, mdate_format=date_format)
        if progress and df is not None:
            print("1/1: Completed!")
        if save_to_file and df is not None:
            if progress:
                print("Saving to file: {}-حقیقی-حقوقی.csv".format(stock))
            df.to_csv(stock + "-حقیقی-حقوقی.csv", encoding="utf-8-sig")
        return df
    else:
        if isinstance(stock, str):
            if progress:
                print("1/1: Getting historical real/legal of {}".format(stock))
            stock = characters.ar_to_fa(stock).strip("\u200c").strip()
            df = _get_stock_RL(stock_name=stock, mstart=start, mend=end, mvalues=values, mtse_format=tse_format,
                               moutput_type=output_type, mdate_format=date_format)
            if progress and df is not None:
                print("1/1: Completed!")
            if save_to_file and df is not None:
                if progress:
                    print("Saving to file: {}-حقیقی-حقوقی.csv".format(stock))
                df.to_csv(stock + "-حقیقی-حقوقی.csv", encoding="utf-8-sig")
            return df
        elif isinstance(stock, list):
            n = 1
            df_dict = {}
            file_name_str = ''
            for stk in stock:
                if progress:
                    print("{}/{}: Getting historical real/legal of {}".format(n, len(stock), stk))
                stk = characters.ar_to_fa(stk).strip("\u200c").strip()
                df = _get_stock_RL(stock_name=stk, mstart=start, mend=end, mvalues=values, mtse_format=tse_format,
                                   moutput_type=output_type, mdate_format=date_format)
                if df is not None:
                    file_name_str += "-" + stk
                    df_dict[stk] = df
                else:
                    print("{} not Found!".format(stk))
                n += 1
            if progress:
                print('{}/{} Completed!'.format(len(stock), len(stock)))

            if len(list(df_dict.keys())) == 0:
                print('None of the entered stocks exist!!')
                return None
            elif len(list(df_dict.keys())) == 1:
                df = df_dict[list(df_dict.keys())[0]]
                if save_to_file and df is not None:
                    if progress:
                        print("Saving to file: {}-حقیقی-حقوقی.csv".format(file_name_str[1:]))
                    df.to_csv(file_name_str[1:] + "-حقیقی-حقوقی.csv", encoding="utf-8-sig")
                return df
            else:
                df = pd.concat(df_dict, axis=1)
                if multi_stock_drop:
                    df.dropna(inplace=True)
                if save_to_file and df is not None:
                    if progress:
                        print("Saving to file: {}-حقیقی-حقوقی.csv".format(file_name_str[1:]))
                    df.to_csv(file_name_str[1:] + "-حقیقی-حقوقی.csv", encoding="utf-8-sig")
                return df



class Settings:
    def __init__(self):
        self.headers = {
            'User-Agent': 'Mozilla/108.0.1 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36'}
        self.en_weekdays = {0: "Monday", 1: "Tuesday", 2: "Wednesday", 3: "Thursday", 4: "Friday", 5: "Saturday",
                            6: "Sunday"}
        self.fa_weekdays = {0: "دوشنبه", 1: "سه شنبه", 2: "چهارشنبه", 3: "پنجشنبه", 4: "جمعه", 5: "شنبه", 6: "یکشنبه"}

        self.index_names = ["شاخص کل", "شاخص هم وزن", "شاخص کل هم وزن", "شاخص هموزن", "شاخص کل هموزن",
                            "شاخص کل هم\u200cوزن",
                            "شاخص قیمت وزنی-ارزشی", "شاخص قیمت هم وزن", "شاخص قیمت هموزن", "شاخص قیمت هم\u200cوزن",
                            "شاخص شناور آزاد", "شاخص شناور", "شاخص بازار اول", "شاخص بازار دوم", "شاخص صنعت",
                            "شاخص 30 شرکت بزرگ", "شاخص سی شرکت بزرگ", "شاخص ۳۰ شرکت بزرگ",
                            "شاخص ۵۰ شرکت فعال", "شاخص پنجاه شرکت فعال", "شاخص 50 شرکت فعال",
                            "شاخص 50 شرکت فعال بورس", "شاخص پنجاه شرکت فعال بورس", "شاخص ۵۰ شرکت فعال بورس"]

        self.url_search = 'http://cdn.tsetmc.com/api/Instrument/GetInstrumentSearch/{}'
        self.url_detail = 'http://old.tsetmc.com/Loader.aspx?Partree=15131M&i={}'
        self.url_stock_list = 'http://old.tsetmc.com/Loader.aspx?ParTree=151114'
        self.url_price_history = 'http://old.tsetmc.com/tsev2/data/Export-txt.aspx?t=i&a=1&b=0&i={}'
        self.url_index_history = 'http://old.tsetmc.com/tsev2/chart/data/IndexFinancial.aspx?i={}&t=ph'
        self.url_client_type = 'http://old.tsetmc.com/tsev2/data/clienttype.aspx?i={}'



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()


def readme_ok():
    with open("README.md", encoding="utf-8") as f:
        return f.read()


def history_ok():
    with open("HISTORY.rst", encoding="utf-8") as n:
        return n.read()


requirements = ['requests', 'pandas', 'persiantools', 'lxml']

test_requirements = ['pytest>=3', ]

setup(
    author="Mohsen Alipour",
    author_email='alipour@algotik.ir',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    description="Get all data from TSETMC.com (Tehran Bourse).",
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme_ok() + '\n\n' + history_ok(),
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords='algotik_tse',
    name='algotik_tse',
    packages=find_packages(include=['algotik_tse', 'algotik_tse.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/mohsenalipour/algotik_tse',
    version='0.2.4',
    zip_safe=False,
)
